import { readCSV } from 'https://deno.land/x/csv/mod.ts';
import { DateTime, datetime } from 'https://deno.land/x/ptera/mod.ts';
import { EOL } from 'https://deno.land/std/fs/eol.ts';
import { brightCyan, yellow, gray } from 'https://deno.land/std@0.117.0/fmt/colors.ts';

if (!Deno.args.length) {
  console.warn('Please provide a filename');
} else {
  const fileExists = await doesFileExist(Deno.args[0]);
  if (fileExists !== true) {
    console.error(fileExists);
    Deno.exit(-1);
  }
}

let skip: number;
skip = Number.parseInt(Deno.args[1] ?? '0', 10);
if (isNaN(skip)) {
  console.error(`Invalid skip argument "${Deno.args[2]}".`);
  Deno.exit(-1);
}

const file = await Deno.open(Deno.args[0], { read: true, write: false });
// really starts at one because it's incremented right away in loop
let rowIndex = 0;

for await (const row of readCSV(file, { lineSeparator: Deno.build.os === 'windows' ? EOL.CRLF : EOL.LF })) {
  rowIndex++;
  let entry: WotdEntry|undefined;
  let word: string|undefined = undefined;
  let date: DateTime|undefined;
  for await (const cell of row) {
    if (!cell || rowIndex <= skip) {
      break;
    } else if (!date) {
      date = parseDate(cell);
      if (!date) {
        console.error(`Parsing error! Invalid date "${cell}" in row ${rowIndex}.`);
        break;
      }
    } else if (!word) {
      word = cell;
    } else if (!entry) {
      entry = { word, pronunciation: cell, definitions: [] };
    } else {
      if (['Noun', 'Verb', 'Adjective', 'Prefix'].includes(cell)) {
        entry.definitions.push({ part: cell, definitions: [] });
      } else if (entry.definitions.length) {
        entry.definitions[entry.definitions.length - 1].definitions.push(cell);
      } else {
        console.error(`Parsing error! Missing a part of speech in row ${rowIndex}.`);
        Deno.exit(-1);
      }
    }
  }
  if (entry && date) {
    const filename = date.format('YYYY.MM.dd');
    await Deno.writeTextFile(filename + '.json', JSON.stringify(entry));
    console.info(`Wrote ${brightCyan(entry.word)} to ${yellow(filename) + gray('.json')}`);
  }
}

async function doesFileExist(filename: string): Promise<true|string> {
  try {
    const stat = await Deno.stat(filename);
    if (stat.isFile) {
      return true;
    } else if (stat.isDirectory) {
      return 'This is a directory. Need a file';
    } else { // isSymlink
      return 'This is a symbolic link. Need a file';
    }
  } catch (e) {
    if (e instanceof Deno.errors.NotFound) {
      return 'File not found.';
    } else if (e instanceof Deno.errors.PermissionDenied) {
      return 'Permission denied to access that file.';
    } else {
      return `Error: ${e.name} (${e.code})`;
    }
  }
}

function parseDate(date: string) {
  const parsed = datetime().parse(date, 'M/d/YYYY');
  return parsed.isValid() ? parsed : undefined;
}

type WotdEntry = {
  word: string;
  pronunciation: string;
  definitions: {
    part: string;
    definitions: string[];
  }[];
}
